@@ -18,6 +18,7 @@
 #include <ucommon/export.h>
 #include <sipwitch/process.h>
 #include <sipwitch/service.h>
+#include <sipwitch/modules.h>
 #include <ctype.h>
 #include <errno.h>
 #include <stdarg.h>
@@ -149,10 +150,6 @@ void service::callback::errlog(errlevel_t level, const char *text)
 {
 }
 
-void service::callback::period(long slice)
-{
-}
-
 void service::callback::cdrlog(cdr *cdr)
 {
 }
@@ -531,7 +528,15 @@ bool service::load(FILE *fp, keynode *node)
 				goto exit;
 
 			bp = strchr(cp, '<');
-			ep = strchr(cp, '>');
+			if(bp == cp && String::equal(bp, "<!--", 4)) {
+				ep = strstr(cp, "-->");
+				if(ep) {
+					cp = ep + 3;
+					break;
+				}
+			}
+			else
+				ep = strchr(cp, '>');
 			if(!ep && bp == cp)
 				break;
 			if(!bp ) {
@@ -709,7 +714,7 @@ void service::dumpfile(const char *uid)
 	fclose(fp);
 }
 
-void service::period(long slice)
+bool service::period(long slice)
 {
 	assert(slice > 0);
 
@@ -720,7 +725,7 @@ void service::period(long slice)
 	time(&now);
 	next = ((periodic / slice) + 1l) * slice;
 	if(now < next)
-		return;
+		return false;
 
 	next = (now / slice) * slice;
 
@@ -741,11 +746,12 @@ void service::period(long slice)
 	if(fp)
 		fclose(fp);
 
-	linked_pointer<service::callback> cb = getModules();
+	linked_pointer<modules::sipwitch> cb = service::getModules();
 	while(is(cb)) {
 		cb->period(slice);
 		cb.next();
 	}
+	return true;
 }
 
 void service::snapshot(const char *uid)
